import json
import os
import pathlib
import platform
import secrets
import tempfile
from pathlib import Path
from typing import Dict, List, Optional, cast

import pydantic
import strenum
import tomlkit
from pydantic import BaseModel, Extra, Field, HttpUrl


class Source(strenum.StrEnum):
    Mikan = "mikan_project"
    BangumiMoe = "bangumi_moe"
    Dmhy = "dmhy"


IS_WINDOWS = platform.system() == "Windows"


def get_bgmi_home() -> Path:
    h = os.environ.get("BGMI_PATH")

    if h:
        return Path(h)

    if IS_WINDOWS:
        home_dir = os.environ.get("USERPROFILE") or os.environ.get("HOME") or tempfile.gettempdir()
        return Path(home_dir).joinpath(".bgmi")

    return Path(os.environ.get("HOME", "/tmp")).joinpath(".bgmi")


BGMI_PATH = get_bgmi_home().absolute()

CONFIG_FILE_PATH = BGMI_PATH / "config.toml"


class BaseSetting(BaseModel):
    class Config:
        validate_assignment = True
        extra = Extra.ignore


class Aria2Config(BaseSetting):
    rpc_url = os.getenv("BGMI_ARIA2_RPC_URL") or "http://127.0.0.1:6800/rpc"
    rpc_token = os.getenv("BGMI_ARIA2_RPC_TOKEN") or "token:"


class TransmissionConfig(BaseSetting):
    rpc_host: str = os.getenv("BGMI_TRANSMISSION_RPC_HOST") or "127.0.0.1"
    rpc_port: int = int(os.getenv("BGMI_TRANSMISSION_RPC_PORT") or "9091")
    rpc_username: str = os.getenv("BGMI_TRANSMISSION_RPC_USERNAME") or "your_username"
    rpc_password: str = os.getenv("BGMI_TRANSMISSION_RPC_PASSWORD") or "your_password"
    rpc_path: str = os.getenv("BGMI_TRANSMISSION_RPC_PATH") or "/transmission/rpc"


class QBittorrentConfig(BaseSetting):
    rpc_host: str = os.getenv("BGMI_QBITTORRENT_RPC_HOST") or "127.0.0.1"
    rpc_port: int = int(os.getenv("BGMI_QBITTORRENT_RPC_PORT") or "8080")
    rpc_username: str = os.getenv("BGMI_QBITTORRENT_RPC_USERNAME") or "admin"
    rpc_password: str = os.getenv("BGMI_QBITTORRENT_RPC_PASSWORD") or "adminadmin"
    category: str = os.getenv("BGMI_QBITTORRENT_RPC_CATEGORY") or ""


class DelugeConfig(BaseSetting):
    rpc_url: HttpUrl = os.getenv("BGMI_DELUGE_RPC_URL") or "http://127.0.0.1:8112/json"  # type: ignore
    rpc_password: str = os.getenv("BGMI_DELUGE_RPC_PASSWORD") or "deluge"


class HTTP(BaseSetting):
    admin_token: str = Field(
        default_factory=lambda: os.getenv("BGMI_HTTP_ADMIN_TOKEN") or secrets.token_urlsafe(12),
        description="webui admin token",
    )
    danmaku_api_url: str = Field(
        os.getenv("BGMI_HTTP_DANMAKU_API_URL") or "",
        description="danmaku api url, https://github.com/DIYgod/DPlayer#related-projects",
    )
    serve_static_files: bool = Field(
        bool(os.getenv("BGMI_HTTP_SERVE_STATIC_FILES")), description="use tornado serving video files"
    )


class Config(BaseSetting):
    data_source: Source = Field(
        os.getenv("BGMI_DATA_SOURCE") or Source.BangumiMoe, description="data source"
    )  # type: ignore
    download_delegate: str = Field(os.getenv("BGMI_DOWNLOAD_DELEGATE") or "aria2-rpc", description="download delegate")

    tmp_path: Path = Path(os.getenv("BGMI_TMP_PATH") or str(BGMI_PATH.joinpath("tmp")))

    proxy: str = cast(str, os.getenv("BGMI_PROXY") or "")

    @property
    def log_path(self) -> Path:
        return self.tmp_path.joinpath("bgmi.log")

    save_path: Path = Field(
        Path(os.getenv("BGMI_SAVE_PATH") or str(BGMI_PATH.joinpath("bangumi"))), description="bangumi save path"
    )
    front_static_path: Path = Path(os.getenv("BGMI_FRONT_STATIC_PATH") or str(BGMI_PATH.joinpath("front_static")))

    db_path: pathlib.Path = Path(os.getenv("BGMI_DB_PATH") or str(BGMI_PATH.joinpath("bangumi.db")))
    script_path: pathlib.Path = Path(os.getenv("BGMI_SCRIPT_PATH") or str(BGMI_PATH.joinpath("scripts")))
    tools_path: pathlib.Path = Path(os.getenv("BGMI_TOOLS_PATH") or str(BGMI_PATH.joinpath("tools")))

    max_path: int = 3

    bangumi_moe_url: HttpUrl = Field(
        os.getenv("BGMI_BANGUMI_MOE_URL") or "https://bangumi.moe", description="Setting bangumi.moe url"
    )  # type: ignore
    share_dmhy_url: HttpUrl = Field(
        os.getenv("BGMI_SHARE_DMHY_URL") or "https://share.dmhy.org", description="Setting share.dmhy.org url"
    )  # type: ignore
    mikan_url: HttpUrl = Field(
        os.getenv("BGMI_MIKAN_URL") or "https://mikanani.me", description="Setting mikanani.me url"
    )  # type: ignore

    mikan_username: str = os.getenv("BGMI_MIKAN_USERNAME") or ""
    mikan_password: str = os.getenv("BGMI_MIKAN_PASSWORD") or ""

    http: HTTP = HTTP()

    # language
    lang: str = os.getenv("BGMI_LANG") or "zh_cn"

    aria2: Aria2Config = Aria2Config()
    transmission: TransmissionConfig = TransmissionConfig()
    qbittorrent: QBittorrentConfig = QBittorrentConfig()
    deluge: DelugeConfig = DelugeConfig()

    enable_global_include_keywords: bool = False
    global_include_keywords: List[str] = ["1080"]

    enable_global_filters: bool = Field(True, description="enable global filter")
    global_filters: List[str] = Field(
        ["Leopard-Raws", "hevc", "x265", "c-a Raws", "U3-Web"], description="Global exclude keywords"
    )

    save_path_map: Dict[str, Path] = Field(default_factory=dict, description="per-bangumi save path")

    def save(self) -> None:
        s = tomlkit.dumps(json.loads(self.json()))

        CONFIG_FILE_PATH.write_text(s, encoding="utf8")


def pydantic_to_toml(obj: pydantic.BaseModel) -> tomlkit.TOMLDocument:
    doc = tomlkit.document()

    d = obj.dict()

    for name, field in obj.__fields__.items():
        if issubclass(field.type_, BaseModel):
            doc.add(name, pydantic_to_toml(getattr(obj, name)))  # type: ignore
            continue

        value = d[name]

        if isinstance(value, Path):
            item = tomlkit.item(str(value))
        else:
            item = tomlkit.item(value)  # type: ignore

        desc: Optional[str] = field.field_info.description
        if desc:
            item.comment(desc)

        doc.add(name, item)

    return doc


if CONFIG_FILE_PATH.exists():
    try:
        cfg = Config.parse_obj(tomlkit.loads(CONFIG_FILE_PATH.read_text(encoding="utf8")).unwrap())
    except pydantic.ValidationError as e:
        print("配置文件错误，请手动编辑配置文件后重试")
        print("配置文件位置：", CONFIG_FILE_PATH)
        print(e)
        raise SystemExit
else:
    cfg = Config()


def print_config() -> str:
    return tomlkit.dumps(json.loads(cfg.json()))


def write_default_config() -> None:
    if not CONFIG_FILE_PATH.exists():
        doc = pydantic_to_toml(Config())

        CONFIG_FILE_PATH.write_text(tomlkit.dumps(doc))


if __name__ == "__main__":
    Config().save()



import datetime
import os
import time
from collections import defaultdict
from typing import List, Optional
from xml.etree import ElementTree

import bs4
from bs4 import BeautifulSoup
from strsimpy.normalized_levenshtein import NormalizedLevenshtein

from bgmi.config import cfg
from bgmi.session import session as requests
from bgmi.utils import parse_episode, print_info
from bgmi.website.base import BaseWebsite
from bgmi.website.model import Episode, SubtitleGroup, WebsiteBangumi

server_root = f"{cfg.mikan_url.rstrip('/')}/"
login_url = f"{server_root}Account/Login"

_COVER_URL = server_root[:-1]

# Example: /Home/ExpandEpisodeTable?bangumiId=2242&subtitleGroupId=34&take=65
bangumi_episode_expand_api = f"{server_root}Home/ExpandEpisodeTable"

_CN_WEEK = {
    "星期日": "Sun",
    "星期一": "Mon",
    "星期二": "Tue",
    "星期三": "Wed",
    "星期四": "Thu",
    "星期五": "Fri",
    "星期六": "Sat",
    "OVA": "Unknown",
}

_DAY_OF_WEEK = {
    0: "星期日",
    1: "星期一",
    2: "星期二",
    3: "星期三",
    4: "星期四",
    5: "星期五",
    6: "星期六",
    8: "OVA",
}


def get_weekly_bangumi():
    """
    network
    """
    r = get_text(server_root)
    soup = bs4.BeautifulSoup(r, "html.parser")
    for day_of_week in [x for x in range(0, 9) if x != 7]:
        d = soup.find("div", attrs={"class": "sk-bangumi", "data-dayofweek": str(day_of_week)})
        if d:
            try:
                yield _CN_WEEK[_DAY_OF_WEEK[day_of_week]], d
            except KeyError:
                pass


def parse_episodes(content, bangumi_id, subtitle_list=None) -> List[Episode]:
    result = []
    soup = BeautifulSoup(content, "html.parser")
    container = soup.find("div", class_="central-container")  # type:bs4.Tag
    episode_container_list = {}
    expand_subtitle_map = {}
    for tag in container.contents:
        if not hasattr(tag, "attrs"):
            continue

        class_names = tag.attrs.get("class")
        if class_names is not None and "episode-expand" in class_names:
            expand_subtitle_map[tag.attrs.get("data-subtitlegroupid", None)] = True

        subtitle_id = tag.attrs.get("id", False)
        if subtitle_list:
            if subtitle_id in subtitle_list:
                episode_container_list[tag.attrs.get("id", None)] = tag.find_next_sibling("table")
        else:
            if subtitle_id:
                episode_container_list[tag.attrs.get("id", None)] = tag.find_next_sibling("table")

    for subtitle_id, container in episode_container_list.items():
        _container = container
        if subtitle_id in expand_subtitle_map:
            expand_r = requests.get(
                bangumi_episode_expand_api,
                params={
                    "bangumiId": bangumi_id,
                    "subtitleGroupId": subtitle_id,
                    "take": 200,
                },
            ).text
            expand_soup = BeautifulSoup(expand_r, "html.parser")
            _container = expand_soup.find("table")

        for tr in _container.find_all("tr")[1:]:
            title = tr.find("a", class_="magnet-link-wrap").text
            time_string = tr.find_all("td")[2].string
            result.append(
                Episode(
                    **{
                        "download": server_root[:-1] + tr.find_all("td")[-1].find("a").attrs.get("href", ""),
                        "subtitle_group": str(subtitle_id),
                        "title": title,
                        "episode": parse_episode(title),
                        "time": int(time.mktime(time.strptime(time_string, "%Y/%m/%d %H:%M"))),
                    }
                )
            )

    return result


def parser_day_bangumi(soup) -> List[WebsiteBangumi]:
    """

    :param soup:
    :type soup: bs4.Tag
    """
    li = []
    for s in soup.find_all("li"):
        url = s.select_one("a")
        span = s.find("span")
        if url:
            name = url["title"]
            url = url["href"]
            bangumi_id = url.split("/")[-1]
            s.find("li")
            li.append(WebsiteBangumi(name=name, keyword=bangumi_id, cover=_COVER_URL + span["data-src"]))
    return li


def mikan_login():
    r = requests.get(login_url)
    soup = BeautifulSoup(r.text, "html.parser")
    token = soup.find("input", attrs={"name": "__RequestVerificationToken"})["value"]

    if os.environ.get("DEBUG", False):  # pragma: no cover
        print(login_url)

    r = requests.post(
        login_url,
        data={
            "UserName": cfg.mikan_username,
            "Password": cfg.mikan_password,
            "__RequestVerificationToken": token,
        },
        headers={"Referer": server_root},
        allow_redirects=False,
    )

    if "&#x767B;&#x5F55;&#x5931;&#x8D25;&#xFF0C;&#x8BF7;&#x91CD;&#x8BD5;" in r.text:  # 实际为 "登录失败，请重试"
        raise ValueError("mikan login failed with wrong username or password")


def get_text(url, params=None):
    if os.environ.get("DEBUG", False):  # pragma: no cover
        print(url, params)

    if not cfg.mikan_username or not cfg.mikan_password:
        return requests.get(url, params=params).text

    for _ in range(2):
        r = requests.get(url, params=params)
        if r.headers.get("content-type").startswith("text/html"):
            if "退出" in r.text:
                return r.text
            else:
                mikan_login()
        else:
            return r.text

    raise ValueError("mikan login failed")


class Mikanani(BaseWebsite):
    def parse_bangumi_details_page(self, r):
        subtitle_groups = defaultdict(dict)

        soup = BeautifulSoup(r, "html.parser")

        # info
        bangumi_info = {"status": 0}
        left_container = soup.select_one("div.pull-left.leftbar-container")
        title = left_container.find("p", class_="bangumi-title")
        day = title.find_next_sibling("p", class_="bangumi-info")
        bangumi_info["name"] = title.text
        bangumi_info["update_time"] = _CN_WEEK[day.text[-3:]]

        ######
        soup = BeautifulSoup(r, "html.parser")
        container = soup.find("div", class_="central-container")
        episode_container_list = {}
        for tag in container.contents:
            if not hasattr(tag, "attrs"):
                continue
            subtitle_id = tag.attrs.get("id", False)
            if subtitle_id:
                episode_container_list[tag.attrs.get("id")] = tag.find_next_sibling("table")

        for subtitle_id, container in episode_container_list.items():
            subtitle_groups[str(subtitle_id)]["episode"] = []
            for tr in container.find_all("tr")[1:]:
                title = tr.find("a", class_="magnet-link-wrap").text
                time_string = tr.find_all("td")[2].string
                subtitle_groups[str(subtitle_id)]["episode"].append(
                    {
                        "download": tr.find("a", class_="magnet-link").attrs.get("data-clipboard-text", ""),
                        "subtitle_group": str(subtitle_id),
                        "title": title,
                        "episode": self.parse_episode(title),
                        "time": int(time.mktime(time.strptime(time_string, "%Y/%m/%d %H:%M"))),
                    }
                )

        ######
        nr = []
        dv = soup.find("div", class_="leftbar-nav")
        li_list = dv.ul.find_all("li")
        for li in li_list:
            a = li.find("a")
            subtitle = {
                "id": a.attrs["data-anchor"][1:],
                "name": a.text,
            }
            nr.append(subtitle)

        bangumi_info["subtitle_group"] = [SubtitleGroup(**x) for x in nr]
        return bangumi_info

    def search_by_tag(self, tag: str, subtitle: Optional[str] = None, count: Optional[int] = None) -> List[Episode]:
        r = get_text(server_root + "Home/Search", params={"searchstr": tag})
        s = BeautifulSoup(r, "html.parser")
        animate = s.find_all("div", attrs={"class": "an-info-group"})[0]
        animate_name = animate.text.strip()
        animate_link = animate.parent.parent.attrs["href"]
        animate_id = animate_link.split("/")[-1]
        print_info(f"Matched animate: {animate_name} ({animate_id})")
        animate_link = animate_link.lstrip("/")

        r = get_text(server_root + animate_link)
        s = BeautifulSoup(r, "html.parser")

        lowest_distance = 1.0
        best_sim_match_group = None

        normalized_levenshtein = NormalizedLevenshtein()

        subgroup_list = s.find_all("div", attrs={"class": "subgroup-text"})
        for subgroup in subgroup_list:
            subgroup_names = []
            subgroup_links = []
            sub_info = {}

            for href_ele in subgroup.find_all("a", href=True):
                link = href_ele["href"]

                subgroup_link_prefix = "/Home/PublishGroup/"
                rss_link_prefix = "/RSS/Bangumi"
                if link:
                    if link.startswith(subgroup_link_prefix):
                        subgroup_names.append(href_ele.text)
                        subgroup_links.append(link[len(subgroup_link_prefix) :])

                    if link.startswith(rss_link_prefix):
                        req: str = link[len(rss_link_prefix) + 1 :]
                        for r in req.split("&"):
                            key, val = r.split("=", maxsplit=1)
                            sub_info[key] = val
            if not subgroup_names:
                continue

            if subtitle:
                cmp_text = " ".join(subgroup_names)
                sim_distance = normalized_levenshtein.distance(cmp_text, subtitle)
                if sim_distance < lowest_distance:
                    lowest_distance = sim_distance
                    best_sim_match_group = (subgroup, subgroup_names, subgroup_links, sub_info)
            else:
                best_sim_match_group = (subgroup, subgroup_names, subgroup_links, sub_info)
                break

        if not best_sim_match_group:
            return []
        subgroup, subgroup_names, subgroup_links, sub_info = best_sim_match_group
        bangumiId, subgroupid = sub_info["bangumiId"], sub_info["subgroupid"]
        rss_url = f"{server_root}RSS/Bangumi?bangumiId={bangumiId}&subgroupid={subgroupid}"

        subtitle_group = " ".join(subgroup_names)
        if subtitle:
            print_info(f"Matched subtitle: {subtitle_group} ({subgroupid})")
        else:
            print_info(f"Use first subtitle: {subtitle_group} ({subgroupid})")

        r = get_text(rss_url)
        rss_root = ElementTree.fromstring(r)

        result = []
        for item in rss_root[0].findall("item"):
            link_el = item.find("link")
            link = link_el.text if link_el is not None else None
            title_el = item.find("title")
            title = title_el.text if title_el is not None else None

            xmlns = "{" + server_root + "0.1/}"
            torrent = item.find(f"{xmlns}torrent")
            pub_date_el = torrent.find(f"{xmlns}pubDate") if torrent is not None else None
            pub_date = pub_date_el.text if pub_date_el is not None else None

            if link and title and pub_date:
                pub_date = pub_date.split(".")[0]
                pub_date_ts = int(datetime.datetime.strptime(pub_date, "%Y-%m-%dT%H:%M:%S").timestamp())

                result.append(
                    Episode(
                        download=link,
                        name=animate_name,
                        title=title,
                        subtitle_group=subtitle_group,
                        episode=self.parse_episode(title),
                        time=pub_date_ts,
                    )
                )
        result = result[::-1]
        return result

    def search_by_keyword(self, keyword, count=None):
        result = []
        r = get_text(server_root + "Home/Search", params={"searchstr": keyword})
        s = BeautifulSoup(r, "html.parser")
        td_list = s.find_all("tr", attrs={"class": "js-search-results-row"})
        for tr in td_list:
            title = tr.find("a", class_="magnet-link-wrap").text
            time_string = tr.find_all("td")[2].string
            result.append(
                Episode(
                    **{
                        "download": tr.find("a", class_="magnet-link").attrs.get("data-clipboard-text", ""),
                        "name": keyword,
                        "title": title,
                        "episode": self.parse_episode(title),
                        "time": int(time.mktime(time.strptime(time_string, "%Y/%m/%d %H:%M"))),
                    }
                )
            )
        return result

    def fetch_episode_of_bangumi(self, bangumi_id, max_page=cfg.max_path, subtitle_list=None):
        r = get_text(server_root + f"Home/Bangumi/{bangumi_id}")
        return parse_episodes(r, bangumi_id, subtitle_list)

    def fetch_bangumi_calendar(self) -> List[WebsiteBangumi]:
        bangumi_list = []
        for update_time, day in get_weekly_bangumi():
            for obj in parser_day_bangumi(day):
                obj.update_time = update_time
                obj.cover = obj.cover.split("?")[0]
                bangumi_list.append(obj)
        return bangumi_list

    def fetch_single_bangumi(
        self,
        bangumi_id: str,
        subtitle_list: Optional[List[str]] = None,
        max_page: int = 0,
    ) -> Optional[WebsiteBangumi]:
        html = get_text(server_root + f"Home/Bangumi/{bangumi_id}")
        info = self.parse_bangumi_details_page(html)
        return WebsiteBangumi(
            name=info["name"],
            keyword=bangumi_id,
            status=info["status"],
            update_time=info["update_time"],
            subtitle_group=info["subtitle_group"],
            episodes=parse_episodes(html, bangumi_id, subtitle_list),
        )



import asyncio
from itertools import chain
from typing import Any, List

import tornado.httpserver
import tornado.ioloop
import tornado.options
import tornado.routing
import tornado.template
import tornado.web
from tornado.options import define, options

from bgmi.config import IS_WINDOWS, cfg
from bgmi.front.admin import API_MAP_GET, API_MAP_POST, AdminApiHandler, UpdateHandler
from bgmi.front.index import BangumiListHandler, IndexHandler
from bgmi.front.resources import BangumiHandler, CalendarHandler, RssHandler

define("port", default=8888, help="listen on the port", type=int)
define("address", default="0.0.0.0", help="binding at given address", type=str)


def make_app() -> tornado.web.Application:
    settings = {
        "autoreload": True,
        "gzip": True,
        "debug": True,
    }
    api_actions = "|".join(chain(API_MAP_GET.keys(), API_MAP_POST.keys()))

    handlers: List[Any] = [
        (r"^/api/(old|index)", BangumiListHandler),
        (r"^/resource/feed.xml$", RssHandler),
        (r"^/resource/calendar.ics$", CalendarHandler),
        (r"^/api/update", UpdateHandler),
        (rf"^/api/(?P<action>{api_actions})", AdminApiHandler),
    ]

    if cfg.http.serve_static_files:
        handlers.extend(
            [
                (r"/bangumi/(.*)", tornado.web.StaticFileHandler, {"path": cfg.save_path}),
                (
                    r"^/(.*)$",
                    tornado.web.StaticFileHandler,
                    {"path": cfg.front_static_path, "default_filename": "index.html"},
                ),
            ]
        )
    else:
        handlers.extend(
            [
                (r"^/bangumi/?(.*)", BangumiHandler),
                (r"^/(.*)$", IndexHandler),
            ]
        )

    return tornado.web.Application(handlers, **settings)  # type: ignore


def main() -> None:
    if IS_WINDOWS:
        asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())

    tornado.options.parse_command_line()
    print(f"BGmi HTTP Server listening on {options.address}:{options.port:d}")
    http_server = tornado.httpserver.HTTPServer(make_app())
    http_server.listen(options.port, address=options.address)
    tornado.ioloop.IOLoop.current().start()


if __name__ == "__main__":
    main()



import qbittorrentapi
from qbittorrentapi import TorrentState

from bgmi.config import cfg
from bgmi.plugin.download import BaseDownloadService, DownloadStatus


class QBittorrentWebAPI(BaseDownloadService):
    def __init__(self):
        self.client = qbittorrentapi.Client(
            host=cfg.qbittorrent.rpc_host,
            port=cfg.qbittorrent.rpc_port,
            username=cfg.qbittorrent.rpc_username,
            password=cfg.qbittorrent.rpc_password,
        )
        self.client.auth_log_in()

    @staticmethod
    def check_config() -> None:
        pass

    def add_download(self, url: str, save_path: str):
        self.client.torrents_add(
            urls=url,
            category=cfg.qbittorrent.category,
            save_path=save_path,
            is_paused=False,
            use_auto_torrent_management=False,
        )

        info = self.client.torrents_info(sort="added_on")

        if info:
            for torrent in info:
                if torrent.save_path == save_path:
                    return torrent.hash
            return info[-1].hash
        return None

    def get_status(self, id: str) -> DownloadStatus:
        torrent = self.client.torrents.info(torrent_hashes=id)
        if not torrent:
            return DownloadStatus.not_found
        state_enum: TorrentState = torrent[0].state_enum
        if state_enum.is_complete or state_enum.is_uploading:
            return DownloadStatus.done
        if state_enum.is_errored:
            return DownloadStatus.error
        if state_enum.is_paused:
            return DownloadStatus.not_downloading
        if state_enum.is_downloading or state_enum.is_checking:
            return DownloadStatus.downloading
        return DownloadStatus.error



import datetime
import os
from collections import defaultdict

from icalendar import Calendar, Event

from bgmi.config import cfg
from bgmi.front.base import BaseHandler
from bgmi.lib.constants import BANGUMI_UPDATE_TIME
from bgmi.lib.models import Download, Followed


class BangumiHandler(BaseHandler):
    def get(self, _: str) -> None:
        if os.environ.get("DEV"):  # pragma: no cover
            with open(os.path.join(cfg.save_path, _), "rb") as f:
                self.write(f.read())
                self.finish()
        else:
            self.set_header("Content-Type", "text/html")
            self.write("<h1>BGmi HTTP Service</h1>")
            self.write(
                "<pre>Please modify your web server configure file\n"
                f"to server this path to '{cfg.save_path}'.\n"
                "e.g.\n\n"
                "...\n"
                "autoindex on;\n"
                "location /bangumi {\n"
                f"    alias {cfg.save_path};\n"
                "}\n"
                "...\n\n"
                "If use want to use Tornado to serve static files, please run\n"
                "<code>[http]</code>"
                "<code>serve_static_files = false</code></pre>"
            )
            self.finish()


class RssHandler(BaseHandler):
    def get(self) -> None:
        data = Download.get_all_downloads()
        self.set_header("Content-Type", "text/xml")
        self.render("templates/download.xml", data=data)


class CalendarHandler(BaseHandler):
    def get(self) -> None:
        type_ = self.get_argument("type", None)

        cal = Calendar()
        cal.add("prodid", "-//BGmi Followed Bangumi Calendar//bangumi.ricterz.me//")
        cal.add("version", "2.0")

        data = Followed.get_all_followed()
        data.extend(self.patch_list)

        if type_ is None:
            bangumi = defaultdict(list)

            for j in data:
                bangumi[BANGUMI_UPDATE_TIME.index(j["update_time"]) + 1].append(j["bangumi_name"])

            weekday = datetime.datetime.now().weekday()
            for i, k in enumerate(range(weekday, weekday + 7)):
                if k % 7 in bangumi:
                    for v in bangumi[k % 7]:
                        event = Event()
                        event.add("summary", v)
                        event.add(
                            "dtstart",
                            datetime.datetime.now().date() + datetime.timedelta(i - 1),
                        )
                        event.add(
                            "dtend",
                            datetime.datetime.now().date() + datetime.timedelta(i - 1),
                        )
                        cal.add_component(event)
        else:
            data = [bangumi for bangumi in data if bangumi["status"] == 2]
            for d in data:
                event = Event()
                event.add("summary", "Updated: {}".format(d["bangumi_name"]))
                event.add("dtstart", datetime.datetime.now().date())
                event.add("dtend", datetime.datetime.now().date())
                cal.add_component(event)

        cal.add("name", "Bangumi Calendar")
        cal.add("X-WR-CALNAM", "Bangumi Calendar")
        cal.add("description", "Followed Bangumi Calendar")
        cal.add("X-WR-CALDESC", "Followed Bangumi Calendar")

        self.write(cal.to_ical())
        self.finish()
